
#include "spm4648.h"

Spm4648::Spm4648() {
}

void Spm4648::read(Stream &port) {
  if(micros() - timeout > 4000) { //interval is 21ms
    p = 0;
  }
  while(port.available()>0) {
    timeout = micros();
    int8_t tmp = port.read();
    if(p<length) {
      buf[p++] = tmp;
      if(p == length) {
        Spm4648::parse();
        newData = 1;
        break;
      }
    } else {
      while(port.available()>0)
        port.read();
      Serial.print("SPM: unexpected byte");

      break;
    }
  }
}
    
float Spm4648::getVal(byte x) {
  for(byte i=0; i<idCount; i++) {
    if(ids[i] == x) {
      float raw = (vals[i] - MINVAL) * 2.0 / (MAXVAL-MINVAL) - 1;
      if(raw > 20)
        raw = -1;
      return constrain(raw, -1, 1);
    }
  }
  return 0;
}
void Spm4648::listVal() {
  for(byte i=0; i<idCount; i++) {
    Serial.print(ids[i], HEX);
    Serial.print(F(": "));
    Serial.print(vals[i]);
    Serial.print(F("  \t"));
  }
  Serial.println();
}

bool Spm4648::hasData() {
  if(newData) {
    newData = 0;
    return true;
  }
  return false;
}
bool Spm4648::isConnected() {
  return micros() - timeout < 500000UL;
}

void Spm4648::parse() {
  for(byte i=0; i<length-1; i+=2) {
    byte id = buf[i] & 0b11111000;
    uint16_t val = (buf[i]&7) << 8 | buf[i+1];
    Spm4648::setVal(id, val);
  }
  data[0] = Spm4648::getVal(0x00);//ly
  data[1] = Spm4648::getVal(0x18);//lx
  data[2] = Spm4648::getVal(0x10);//ry
  data[3] = Spm4648::getVal(0x08);//rx
  data[4] = Spm4648::getVal(0x20);//sw
}

void Spm4648::setVal(byte x, uint16_t val) {
  for(byte i=0; i<idCount; i++) {
    if(ids[i] == x) {
      //limit div size
      if(val > MAXSTEP + vals[i])
        vals[i] += MAXSTEP;
      else if(vals[i] >= MAXSTEP && val < vals[i] - MAXSTEP)
        vals[i] -= MAXSTEP;
      else
        vals[i] = val;
      
      return;
    }
  }
  
  if(idCount >= CHANNELS)
    return;
  
  ids[idCount] = x;
  vals[idCount++] = val;
}
